import React, { useEffect, useMemo } from 'react';
import { useFocusRing } from '@react-aria/focus';
import CollapseIcon from './collapse-icon';
import Expand from '../utils/expand';
import { useCollapseContext } from './collapse-context';
import useCurrentState from '../use-current-state';
import useWarning from '../use-warning';
import { useId } from '@react-aria/utils';
import useKeyboard, { KeyCode } from '../use-keyboard';
import { StyledCollapse, StyledCollapseView, StyledCollapseContent } from './collapse.styles';
import clsx from '../utils/clsx';
import withDefaults from '../utils/with-defaults';
import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
const defaultProps = {
  shadow: false,
  divider: true,
  bordered: false,
  showArrow: true,
  animated: true,
  disabled: false,
  preventDefault: true,
  expanded: false
};
const preClass = 'nextui-collapse';

const Collapse = ({
  children,
  title,
  subtitle,
  expanded,
  shadow,
  className,
  divider,
  arrowIcon,
  showArrow,
  disabled,
  onChange,
  bordered,
  contentLeft,
  preventDefault,
  animated: animatedProp,
  borderWeight,
  index,
  ...props
}) => {
  const [visible, setVisible, visibleRef] = useCurrentState(expanded);
  const {
    values,
    divider: groupDivider,
    animated: groupAnimated,
    updateValues
  } = useCollapseContext();
  const {
    isFocusVisible,
    focusProps
  } = useFocusRing();

  if (!title) {
    useWarning('"title" is required.', 'Collapse');
  }

  useEffect(() => {
    if (visible !== expanded) {
      setVisible(expanded);
    }
  }, [expanded]);
  useEffect(() => {
    if (!values.length) return;
    const isActive = !!values.find(item => item === index);
    setVisible(isActive);
  }, [values.join(',')]);
  const ariaLabelledById = useId();
  const ariaControlId = useId();
  const arrowComponent = useMemo(() => {
    if (!showArrow) return null;
    return arrowIcon ? arrowIcon : /*#__PURE__*/_jsx(CollapseIcon, {});
  }, [arrowIcon, showArrow]);
  const withDivider = groupDivider === undefined ? divider : groupDivider;
  const animated = useMemo(() => {
    return groupAnimated === undefined ? animatedProp : groupAnimated;
  }, [groupAnimated, animatedProp]);

  const handleChange = event => {
    if (disabled) return;
    const next = !visibleRef.current;
    setVisible(next);
    updateValues && updateValues(index, next);
    onChange && onChange(event, index, next);
  };

  const {
    bindings
  } = useKeyboard(event => {
    handleChange(event);
  }, [KeyCode.Enter, KeyCode.Space], {
    disableGlobalEvent: true,
    preventDefault
  });
  const getState = useMemo(() => {
    return visible ? 'open' : 'closed';
  }, [visible]);
  return /*#__PURE__*/_jsxs(StyledCollapse, {
    tabIndex: -1,
    shadow: shadow,
    bordered: bordered,
    animated: animated,
    divider: withDivider,
    borderWeight: borderWeight,
    visible: visible,
    "data-state": getState,
    className: clsx(className, preClass, `${preClass}--${getState}`),
    ...props,
    children: [/*#__PURE__*/_jsx(StyledCollapseView, {
      role: "button",
      tabIndex: disabled ? -1 : 0,
      id: ariaLabelledById,
      className: `${preClass}-view`,
      "data-state": getState,
      disabled: disabled,
      "aria-disabled": disabled,
      "aria-expanded": visible,
      "aria-controls": ariaControlId,
      isFocusVisible: isFocusVisible,
      onClick: handleChange,
      ...focusProps,
      ...bindings,
      children: /*#__PURE__*/_jsxs("div", {
        className: clsx(`${preClass}-title-container`),
        children: [contentLeft && /*#__PURE__*/_jsx("div", {
          className: `${preClass}-title-content-left`,
          children: contentLeft
        }), /*#__PURE__*/_jsxs("div", {
          className: `${preClass}-title-content`,
          children: [/*#__PURE__*/React.isValidElement(title) ? title : /*#__PURE__*/_jsx("h3", {
            className: `${preClass}-title`,
            children: title
          }), subtitle && /*#__PURE__*/_jsx("div", {
            className: `${preClass}-subtitle`,
            children: subtitle
          })]
        }), /*#__PURE__*/_jsx("div", {
          className: `${preClass}-title-content-right`,
          children: arrowComponent
        })]
      })
    }), /*#__PURE__*/_jsx(Expand, {
      isExpanded: visible,
      animated: animated,
      children: /*#__PURE__*/_jsx(StyledCollapseContent, {
        role: "region",
        tabIndex: -1,
        id: ariaControlId,
        "aria-labelledby": ariaLabelledById,
        className: `${preClass}-content`,
        children: children
      })
    })]
  });
};

Collapse.toString = () => '.nextui-collapse';

export default withDefaults(Collapse, defaultProps);