import React, { useEffect, useMemo, useState } from 'react';
import ImageSkeleton from './image.skeleton';
import useRealShape from '../use-real-shape';
import useCurrentState from '../use-current-state';
import useResize from '../use-resize';
import { useDOMRef } from '../utils/dom';
import { __DEV__ } from '../utils/assertion';
import { StyledImage, StyledImageContainer } from './image.styles';
import clsx from '../utils/clsx';
import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
const defaultProps = {
  showSkeleton: true,
  autoResize: false,
  objectFit: 'scale-down',
  maxDelay: 3000,
  className: ''
};
export const Image = /*#__PURE__*/React.forwardRef((props, ref) => {
  const {
    src,
    width,
    height,
    showSkeleton: showSkeletonProp,
    className,
    maxDelay,
    autoResize,
    objectFit,
    containerCss,
    css,
    ...otherProps
  } = props;
  const imageRef = useDOMRef(ref);
  const [loading, setLoading] = useState(true);
  const [showSkeleton, setShowSkeleton] = useState(showSkeletonProp);
  const {
    w,
    h
  } = useMemo(() => {
    return {
      w: width ? typeof width === 'number' ? `${width}px` : width : 'auto',
      h: height ? typeof height === 'number' ? `${height}px` : height : 'auto'
    };
  }, [width, height]);
  const [zoomHeight, setZoomHeight, zoomHeightRef] = useCurrentState(h);
  const [shape, updateShape] = useRealShape(imageRef);
  const showAnimation = showSkeletonProp && !!width && !!height;

  const onImageLoaded = () => {
    setLoading(false);
  };

  useEffect(() => {
    if (!imageRef.current) return;

    if (imageRef.current.complete) {
      setLoading(false);
      setShowSkeleton(false);
    }
  });
  useEffect(() => {
    const timer = setTimeout(() => {
      if (showAnimation) {
        setShowSkeleton(false);
      }

      clearTimeout(timer);
    }, maxDelay);
    return () => clearTimeout(timer);
  }, [loading]);
  /**
   * On mobile devices, the render witdth may be less than CSS width value.
   * If the image is scaled, set the height manually.
   * This is to ensure the aspect ratio of the image.
   *
   * If the image is auto width, ignore all.
   */

  useEffect(() => {
    if (!autoResize) return;
    const notLoaded = shape.width === 0;
    const isAutoZoom = zoomHeightRef.current === 'auto';
    if (notLoaded || !width || !height) return;

    if (shape.width < width) {
      !isAutoZoom && setZoomHeight('auto');
    } else {
      isAutoZoom && setZoomHeight(h);
    }
  }, [shape, width]);
  useResize(() => {
    if (!autoResize) return;
    updateShape();
  });
  const getState = useMemo(() => {
    return loading ? 'loading' : 'ready';
  }, [loading]);
  return /*#__PURE__*/_jsxs(StyledImageContainer, {
    className: clsx('nextui-image-container', `nextui-image--${getState}`, className),
    "data-state": getState,
    ready: !loading || showSkeleton,
    css: {
      width: w,
      height: zoomHeight,
      ...containerCss
    },
    children: [showSkeleton && /*#__PURE__*/_jsx(ImageSkeleton, {
      opacity: 1
    }), /*#__PURE__*/_jsx(StyledImage, {
      ref: imageRef,
      className: "nextui-image",
      width: width,
      height: height,
      onLoad: onImageLoaded,
      src: src,
      "data-state": getState,
      alt: props.alt || '',
      css: {
        objectFit,
        ...css
      },
      ...otherProps
    })]
  });
});

if (__DEV__) {
  Image.displayName = 'NextUI.Image';
}

Image.toString = () => '.nextui-image';

Image.defaultProps = defaultProps;
export default /*#__PURE__*/React.memo(Image);