import React, { useEffect, useImperativeHandle, useMemo, useRef, useState } from 'react';
import { useLabel } from '@react-aria/label';
import { defaultProps } from './input-props';
import useTheme from '../use-theme';
import useWarning from '../use-warning';
import { StyledInput, StyledInputMainContainer, StyledInputContainer, StyledHelperTextContainer, StyledHelperText, StyledInputWrapper, StyledInputPlaceholder, StyledInputClearButton as InputClearButton, StyledInputBlockLabel as InputBlockLabel, StyledInputLabel as InputLabel, StyledInputContent as InputContent } from './input.styles';
import ClearIcon from '../utils/clear-icon';
import clsx from '../utils/clsx';
import { __DEV__ } from '../utils/assertion';
import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";

const simulateChangeEvent = (el, event) => {
  return { ...event,
    target: el,
    currentTarget: el
  };
};

const preClass = 'nextui-input';
const Input = /*#__PURE__*/React.forwardRef(({
  as: Component = 'input',
  label,
  labelPlaceholder,
  labelLeft,
  labelRight,
  size,
  helperText,
  color,
  helperColor,
  status,
  contentLeft,
  contentRight,
  contentClickable,
  contentLeftStyling,
  contentRightStyling,
  onContentClick,
  initialValue,
  onChange,
  readOnly,
  value,
  onClearClick,
  clearable,
  shadow,
  animated,
  required,
  width: widthProp,
  className,
  onBlur,
  onFocus,
  autoComplete,
  placeholder,
  fullWidth,
  borderWeight,
  disabled,
  bordered,
  underlined,
  rounded,
  css,
  ...props
}, ref) => {
  const inputRef = useRef(null);
  useImperativeHandle(ref, () => inputRef.current);
  const [selfValue, setSelfValue] = useState(initialValue);
  const [hover, setHover] = useState(false);
  const {
    isDark
  } = useTheme();
  const isControlledComponent = useMemo(() => value !== undefined, [value]);
  const inputLabel = useMemo(() => label || labelPlaceholder, [label, labelPlaceholder]);
  const inputPlaceholder = useMemo(() => labelPlaceholder ? '' : placeholder, [placeholder, labelPlaceholder]);
  const width = useMemo(() => {
    if (fullWidth) return '100%';
    if (widthProp) return widthProp;
    return 'initial';
  }, [fullWidth, widthProp]);

  if (underlined && __DEV__) {
    bordered && useWarning('Using underlined and bordered at the same time will have no effect.');
    rounded && useWarning('Using underlined and rounded at the same time will have no effect.');
  }

  const changeHandler = event => {
    if (disabled || readOnly) return;
    setSelfValue(event.target.value);
    onChange && onChange(event);
  };

  const clearHandler = event => {
    event.preventDefault();
    event.stopPropagation();
    event.nativeEvent.stopImmediatePropagation();
    setSelfValue('');
    onClearClick && onClearClick(event);
    /* istanbul ignore next */

    if (!inputRef.current) return;
    const changeEvent = simulateChangeEvent(inputRef.current, event);
    changeEvent.target.value = '';
    onChange && onChange(changeEvent);
    inputRef.current.focus();
  };

  const focusHandler = e => {
    setHover(true);
    onFocus && onFocus(e);
  };

  const blurHandler = e => {
    setHover(false);
    onBlur && onBlur(e);
  };

  const contentClickHandler = (key, e) => {
    if (disabled) return;
    onContentClick && onContentClick(key, e);
  };

  useEffect(() => {
    if (isControlledComponent) {
      setSelfValue(value);
    }
  });
  const isTextarea = useMemo(() => Component === 'textarea', [Component]);
  const controlledValue = isControlledComponent ? {
    value: selfValue
  } : {
    defaultValue: initialValue
  };
  const inputProps = { ...props,
    ...controlledValue
  };
  const {
    labelProps,
    fieldProps
  } = useLabel({ ...inputProps,
    label: inputLabel
  });
  const getState = useMemo(() => {
    return hover ? 'hover' : disabled ? 'disabled' : readOnly ? 'read-only' : selfValue ? 'with-value' : 'normal';
  }, [hover, disabled, readOnly, selfValue]);
  return /*#__PURE__*/_jsxs(StyledInputMainContainer, {
    color: color,
    "data-state": getState,
    helperColor: helperColor,
    borderWeight: borderWeight,
    status: status,
    size: size,
    rounded: rounded,
    disabled: disabled,
    css: {
      width,
      ...css
    },
    className: clsx(`${preClass}-main-container`, `${preClass}-main-container--${getState}`),
    children: [inputLabel && /*#__PURE__*/_jsx(InputBlockLabel, {
      id: labelProps.id,
      className: `${preClass}-block-label`,
      htmlFor: labelProps.htmlFor,
      isTextarea: isTextarea,
      underlined: underlined,
      animated: animated,
      bordered: bordered,
      rounded: rounded,
      hasContentLeft: !!contentLeft,
      withValue: !!selfValue,
      asPlaceholder: !!labelPlaceholder,
      focused: hover,
      children: inputLabel
    }), /*#__PURE__*/_jsx(StyledInputContainer, {
      animated: animated,
      isTextarea: isTextarea,
      underlined: underlined,
      isReadOnly: readOnly,
      focused: hover,
      className: clsx(`${preClass}-container`, `${preClass}-container--${getState}`, {
        [`${preClass}-container--input`]: !isTextarea,
        [`${preClass}-container--textarea`]: isTextarea,
        [`${preClass}-container--read-only`]: readOnly
      }, className),
      children: /*#__PURE__*/_jsxs(StyledInputWrapper, {
        as: inputLabel ? 'div' : 'label',
        animated: animated,
        bordered: bordered,
        shadow: shadow,
        disabled: disabled,
        focused: hover,
        isReadOnly: readOnly,
        underlined: underlined,
        isTextarea: isTextarea,
        className: clsx(`${preClass}-wrapper`, `${preClass}-wrapper--${getState}`, {
          [`${preClass}-wrapper--bordered`]: bordered,
          [`${preClass}-wrapper--underlined`]: underlined,
          [`${preClass}-wrapper--shadow`]: shadow
        }),
        children: [!inputLabel && placeholder && /*#__PURE__*/_jsx(StyledInputPlaceholder, {
          className: `${preClass}-placeholder`,
          children: placeholder
        }), labelLeft && /*#__PURE__*/_jsx(InputLabel, {
          className: `${preClass}-label--left`,
          isDefaultStatus: status === 'default',
          underlined: underlined,
          bordered: bordered,
          isDark: isDark,
          children: labelLeft
        }), contentLeft && /*#__PURE__*/_jsx(InputContent, {
          className: clsx(`${preClass}-content`, `${preClass}-content--left`),
          applyStyles: contentLeftStyling,
          clickable: contentClickable,
          onClick: e => contentClickHandler('left', e),
          children: contentLeft
        }), /*#__PURE__*/_jsx(StyledInput, {
          type: "text",
          as: Component,
          ref: inputRef,
          className: clsx({
            [`${preClass}`]: !isTextarea,
            [`${preClass}-textarea`]: isTextarea,
            [`${preClass}-disabled`]: disabled,
            [`${preClass}-rounded`]: rounded,
            [`${preClass}-${preClass}-right-content`]: contentRight,
            [`${preClass}-left-content`]: contentLeft
          }),
          isTextarea: isTextarea,
          focused: hover,
          bordered: bordered,
          placeholder: inputPlaceholder,
          disabled: disabled,
          readOnly: readOnly,
          required: required,
          onFocus: focusHandler,
          onBlur: blurHandler,
          onChange: changeHandler,
          autoComplete: autoComplete,
          hasLeftContent: !!contentLeft,
          hasRightContent: !!contentRight,
          "data-state": getState,
          "aria-placeholder": inputPlaceholder,
          "aria-readonly": readOnly,
          "aria-required": required,
          "aria-multiline": isTextarea,
          ...inputProps,
          ...fieldProps
        }), clearable && /*#__PURE__*/_jsx(InputClearButton, {
          type: "button",
          className: `${preClass}-clear-button`,
          animated: animated,
          underlined: underlined,
          visible: Boolean(selfValue),
          hasContentRight: !!contentRight,
          disabled: disabled || readOnly,
          onClick: clearHandler,
          children: /*#__PURE__*/_jsx(ClearIcon, {
            fill: "currentColor"
          })
        }), contentRight && /*#__PURE__*/_jsx(InputContent, {
          className: clsx(`${preClass}-content`, `${preClass}-content--right`),
          applyStyles: contentRightStyling,
          clickable: contentClickable,
          onClick: e => contentClickHandler('right', e),
          children: contentRight
        }), labelRight && /*#__PURE__*/_jsx(InputLabel, {
          className: `${preClass}-label--right`,
          isDefaultStatus: status === 'default',
          underlined: underlined,
          bordered: bordered,
          isRight: true,
          isDark: isDark,
          children: labelRight
        })]
      })
    }), /*#__PURE__*/_jsx(StyledHelperTextContainer, {
      className: `${preClass}-helper-text-container`,
      withValue: !!helperText,
      children: helperText && /*#__PURE__*/_jsx(StyledHelperText, {
        className: `${preClass}-helper-text`,
        children: helperText
      })
    })]
  });
});
Input.displayName = 'NextUI.Input';
Input.defaultProps = defaultProps;

Input.toString = () => '.nextui-input';

export default Input;