import React, { useEffect, useMemo, useState } from 'react';
import { useRadioContext } from './radio-context';
import { pickChild } from '../utils/collections';
import useWarning from '../use-warning';
import useKeyboard, { KeyCode } from '../use-keyboard';
import withDefaults from '../utils/with-defaults';
import { useDOMRef } from '../utils/dom';
import { __DEV__ } from '../utils/assertion';
import clsx from '../utils/clsx';
import { StyledRadio, StyledRadioLabel, StyledRadioInput, StyledRadioPoint, StyledRadioDescription } from './radio.styles';
import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
const defaultProps = {
  size: 'md',
  color: 'default',
  textColor: 'default',
  disabled: false,
  preventDefault: true
};
const preClass = 'nextui-radio';
export const Radio = /*#__PURE__*/React.forwardRef((props, ref) => {
  const {
    checked,
    onChange,
    disabled,
    color,
    size,
    textColor,
    value: radioValue,
    preventDefault,
    children,
    ...otherProps
  } = props;
  const [selfChecked, setSelfChecked] = useState(!!checked);
  const domRef = useDOMRef(ref);
  const {
    value: groupValue,
    disabledAll,
    inGroup,
    color: groupColor,
    size: groupSize,
    textColor: textGroupColor,
    updateState
  } = useRadioContext();
  const [withoutDescChildren, DescChildren] = pickChild(children, StyledRadioDescription);

  if (inGroup && __DEV__) {
    if (checked !== undefined) {
      useWarning('Remove props "checked" if in the Radio.Group.', 'Radio');
    }

    if (radioValue === undefined) {
      useWarning('Props "value" must be deinfed if in the Radio.Group.', 'Radio');
    }
  }

  useEffect(() => {
    setSelfChecked(groupValue === radioValue);
  }, [groupValue, radioValue]);
  const isDisabled = useMemo(() => disabled || disabledAll, [disabled, disabledAll]);
  const radioColor = color !== 'default' ? color : groupColor;
  const radioSize = size !== 'md' ? size : groupSize;
  const labelColor = textColor !== 'default' ? textColor : textGroupColor;

  const changeHandler = event => {
    if (isDisabled || inGroup && selfChecked) return;
    const selfEvent = {
      target: {
        checked: !selfChecked
      },
      stopPropagation: event.stopPropagation,
      preventDefault: event.preventDefault,
      nativeEvent: event
    };
    setSelfChecked(!selfChecked);

    if (inGroup) {
      updateState && updateState(radioValue);
    }

    onChange && onChange(selfEvent);
  };

  const {
    bindings
  } = useKeyboard(event => {
    changeHandler(event);
  }, [KeyCode.Enter, KeyCode.Space], {
    disableGlobalEvent: true,
    preventDefault
  });
  useEffect(() => {
    if (checked === undefined) return;
    setSelfChecked(Boolean(checked));
  }, [checked]);
  return /*#__PURE__*/_jsxs(StyledRadio, {
    "aria-checked": selfChecked,
    disabled: isDisabled,
    active: selfChecked,
    size: radioSize,
    color: radioColor,
    ...otherProps,
    ...bindings,
    children: [/*#__PURE__*/_jsx(StyledRadioInput, {
      type: "radio",
      tabIndex: -1,
      ref: domRef,
      value: radioValue,
      checked: selfChecked,
      onChange: changeHandler,
      className: `${preClass}-input`
    }), /*#__PURE__*/_jsxs(StyledRadioLabel, {
      color: labelColor,
      disabled: isDisabled,
      className: `${preClass}-name`,
      children: [/*#__PURE__*/_jsx(StyledRadioPoint, {
        tabIndex: isDisabled ? -1 : 0,
        className: clsx(`${preClass}-point`, {
          [`${preClass}-active`]: selfChecked,
          [`${preClass}-disabled`]: isDisabled
        })
      }), withoutDescChildren]
    }), DescChildren && DescChildren]
  });
});
Radio.defaultProps = defaultProps;

if (__DEV__) {
  Radio.displayName = 'NextUI.Radio';
}

Radio.toString = () => '.nextui-radio';

export default withDefaults(Radio, defaultProps);