import React, { useCallback, useEffect, useMemo, useState } from 'react';
import withDefaults from '../utils/with-defaults';
import useWarning from '../use-warning';
import useKeyboard, { KeyCode } from '../use-keyboard';
import { StyledSwitch, StyledSwitchContainer, StyledSwitchInput, StyledSwitchCircle } from './switch.styles';
import clsx from '../utils/clsx';
import { __DEV__ } from '../utils/assertion';
import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
const defaultProps = {
  disabled: false,
  bordered: false,
  shadow: false,
  squared: false,
  animated: true,
  preventDefault: true,
  initialChecked: false
};
const preClass = 'nextui-switch';

const Switch = ({
  initialChecked,
  checked,
  disabled,
  onChange,
  squared,
  bordered,
  shadow,
  icon,
  iconOn,
  iconOff,
  animated,
  preventDefault,
  ...props
}) => {
  const [selfChecked, setSelfChecked] = useState(initialChecked);

  if (icon && __DEV__ && (iconOn || iconOff)) {
    useWarning('Remove props "icon" if iconOn or iconOff exists.', 'Switch');
  }

  const changeHandle = useCallback(ev => {
    if (disabled) return;
    const selfEvent = {
      target: {
        checked: !selfChecked
      },
      stopPropagation: ev.stopPropagation,
      preventDefault: ev.preventDefault,
      nativeEvent: ev
    };
    setSelfChecked(!selfChecked);
    onChange && onChange(selfEvent);
  }, [disabled, selfChecked, onChange]);
  const {
    bindings
  } = useKeyboard(event => {
    changeHandle(event);
  }, [KeyCode.Enter, KeyCode.Space], {
    disableGlobalEvent: true,
    preventDefault
  });
  const circleIcon = useMemo(() => {
    const hasIcon = icon || iconOn || iconOff;
    const hasIconOn = Boolean(iconOn);
    const hasIconOff = Boolean(iconOff);
    if (!hasIcon) return null;
    if (hasIconOn && selfChecked) return iconOn;
    if (hasIconOff && !selfChecked) return iconOff;
    return hasIcon;
  }, [selfChecked, icon, iconOn, iconOff]);
  useEffect(() => {
    if (checked === undefined) return;
    setSelfChecked(checked);
  }, [checked]);
  const getState = useMemo(() => {
    return selfChecked ? 'checked' : 'unchecked';
  }, [selfChecked]);
  return /*#__PURE__*/_jsxs(StyledSwitchContainer, {
    "data-state": getState,
    disabled: disabled,
    animated: animated,
    ...props,
    children: [/*#__PURE__*/_jsx(StyledSwitchInput, {
      tabIndex: -1,
      type: "checkbox",
      className: clsx(`${preClass}-input`),
      "data-state": getState,
      disabled: disabled,
      checked: selfChecked,
      onChange: changeHandle
    }), /*#__PURE__*/_jsx(StyledSwitch, {
      role: "switch",
      tabIndex: disabled ? -1 : 0,
      checked: selfChecked,
      "aria-checked": selfChecked,
      "aria-disabled": disabled,
      animated: animated,
      disabled: disabled,
      squared: squared,
      bordered: bordered,
      shadow: shadow,
      "data-state": getState,
      className: clsx(preClass, `${preClass}--${getState}`, {
        [`${preClass}-checked`]: selfChecked,
        [`${preClass}-disabled`]: disabled
      }),
      ...bindings,
      children: /*#__PURE__*/_jsx(StyledSwitchCircle, {
        className: `${preClass}-circle`,
        children: circleIcon
      })
    })]
  });
};

Switch.toString = () => '.nextui-switch';

const MemoSwitch = /*#__PURE__*/React.memo(Switch);
export default withDefaults(MemoSwitch, defaultProps);